//
//  Cube4DView.m
//  Cube4D
//
//  Created by Alex Boreskoff on 4/16/06.
//  Copyright (c) 2006, __MyCompanyName__. All rights reserved.
//

#import "Cube4DView.h"
#import <math.h>

static	Vector4D	e1      = { 1, 2, -5, 9 };
static	Vector4D	e2      = { -9, 4, 2, 7 };
static	Vector4D	dir     = { 1, -1, 3, -1 };
static	float		angle1  = 0;
static	float		angle2  = 0;
static	float		step1   = 0.05f;
static	float		step2   = 0.03f;

float	dot ( Vector4D * v1, Vector4D * v2 )
{
	return v1 -> x * v2 -> x + v1 -> y * v2 -> y + v1 -> z * v2 -> z + v1 -> w * v2 -> w;
}

void	mul ( Vector4D * v, float f )
{
	v -> x *= f;
	v -> y *= f;	
	v -> z *= f;	
	v -> w *= f;
}

void	sum ( Vector4D * v1, Vector4D * v2 )
{
	v1 -> x += v2 -> x;
	v1 -> y += v2 -> y;
	v1 -> z += v2 -> z;
	v1 -> w += v2 -> w;
}

void	normalize ( Vector4D * v )
{
	mul ( v, 1.0 / sqrt ( dot ( v, v ) ) );
}

void	orthonormalize ()
{
	float	d      = dot ( &e1, &e2 );
	float	len1Sq = dot ( &e1, &e1 );
	Vector4D temp  = e1;
	
	mul ( &temp, -d / len1Sq );
	sum ( &e2, &temp );
	
	normalize ( &e1 );
	normalize ( &e2 );
}

void	transform ( Vector4D * v, float matrix [4][4], Vector4D * res )
{
	int	i, j;
	
	for ( i = 0; i < 4; i++ )
	{
		res -> c [i] = 0;
		
		for ( j = 0; j < 4; j++ )
			res -> c [i] += v -> c [j] * matrix [i][j];
	}
}

void	drawCube ( NSRect bounds, int brightness, float a1, float a2 )
{
	NSPoint		center;
	NSSize		scale;
	float		matrix [4][4];
	int			i, j;
	Vector4D	b1, b2;
	float		br      = (float) brightness / 255.0f;
	NSBezierPath * path = [NSBezierPath bezierPath];
	
	[[NSColor colorWithDeviceRed: br green: br blue: br alpha: 1.0] set];
	
	center.x     = bounds.origin.x + bounds.size.width  * 0.5;
	center.y     = bounds.origin.y + bounds.size.height * 0.5;
	scale.width  = bounds.size.width  / 45.0;
	scale.height = bounds.size.height / 45.0;
	
	for ( i = 0; i < 4; i++ )
		for ( j = 0; j < 4; j++ )
			matrix [i][j] = 0;
	
	matrix [0][0] =  (float) cos ( a1 );
	matrix [1][1] =  (float) cos ( a1 );
	matrix [0][1] =  (float) sin ( a1 );
	matrix [1][0] = -(float) sin ( a1 );
	matrix [2][2] =  (float) cos ( a2 );
	matrix [3][3] =  (float) cos ( a2 );
	matrix [2][3] =  (float) sin ( a2 );
	matrix [3][2] = -(float) sin ( a2 );
	
	transform ( &e1, matrix, &b1 );
	transform ( &e2, matrix, &b2 );
	
	for ( i = 0; i < 15; i++ )
	{
		Vector4D	v;
		
								// fill v with binary representation of i
		v.w = (i & 1 ? 1.0f : 0.0f);
		v.z = (i & 2 ? 1.0f : 0.0f);
		v.y = (i & 4 ? 1.0f : 0.0f);
		v.x = (i & 8 ? 1.0f : 0.0f);
		
		for ( j = 0; j < 4; j++ )
		{
			Vector4D	   v2 = v;
			Vector4D	   offs, w, w2;
			NSPoint		   start, end;
			
			if ( v.c [j] == 1 )
				continue;
			
			v2.c [j] = 1.0;
			
			offs = dir;
			w    = v;
			w2   = v2;

			mul ( &offs, sin ( angle1 * 0.33f + 4*angle2 * 0.2f ) * 0.1 );
			sum ( &w,    &offs );
			sum ( &w2,   &offs );
			
			start.x = dot ( &w,  &b1 ) / (0.7 + 0.2*v.w);
			start.y = dot ( &w,  &b2 ) / (0.7 + 0.2*v.w);
			end.x   = dot ( &w2, &b1 ) / (0.7 + 0.2*v2.w);
			end.y   = dot ( &w2, &b2 ) / (0.7 + 0.2*v2.w);
					
							// remap projection to bounds
			start.x = center.x + start.x * scale.width;
			start.y = center.y + start.y * scale.height;
			end.x   = center.x + end.x   * scale.width;
			end.y   = center.y + end.y   * scale.height;
			
			[path moveToPoint: start];
			[path lineToPoint: end];
		}	
	}
	
	[path closePath];
	[path stroke];
}

@implementation Cube4DView

- (id)initWithFrame: (NSRect)frame isPreview:(BOOL) isPreview
{
    self = [super initWithFrame:frame isPreview:isPreview];
	
    if (self) 
        [self setAnimationTimeInterval:1/20.0];

    return self;
}

- (void)startAnimation
{
    [super startAnimation];
}

- (void)stopAnimation
{
    [super stopAnimation];
}

- (void)drawRect:(NSRect)rect
{
    [super drawRect:rect];
	
	[[NSColor blackColor] set];
	[[NSBezierPath bezierPathWithRect: rect] fill];
	
	drawCube ( rect, 64,  angle1 - 4 * step1 / 2, angle2 - 4 * step2 / 2 );
	drawCube ( rect, 96,  angle1 - 3 * step1 / 2, angle2 - 3 * step2 / 2 );
	drawCube ( rect, 128, angle1 - 2 * step1 / 2, angle2 - 2 * step2 / 2 );
	drawCube ( rect, 192, angle1 -     step1 / 2, angle2 -     step2 );
	drawCube ( rect, 255, angle1,                 angle2 );
	
	angle1 += step1;
	angle2 += step2;
}

- (void)animateOneFrame
{
	[self setNeedsDisplay: YES];
	
    return;
}

- (BOOL)hasConfigureSheet
{
    return NO;
}

- (NSWindow*)configureSheet
{
    return nil;
}

@end
